#! /bin/sh

#
#	rtt.sh - collect round-trip-time stats
#
#ident "@(#)cricket:PLANIX-1_0_3-V_1:rtt.sh,v 1.2 2002/12/06 05:38:49 woods Exp"

# Usage:
#
# rtt [-c count] [-s size] host|IP#
#
# This script munges the output of ping
# into a format suitable for Cricket's EXEC function.
# 
# In order to measure the latency and loss of a network connection it
# is important to choose a remote host that does not itself suffer
# lack of resources and which will not artificailly demonstrate hight
# latency or packet loss when it is under load.  Cisco (and probably
# most other sophisticated) routers are very bad choices because of
# this reason (unless they are indeed almost 100% idle).
# 
# the ouptput consists of:
#
# line	value	description
#
#  0	1.679	minimum rtt in ms
#  1	1.679	average rtt in ms
#  2	1.679	maximum rtt in ms
#  3	0.0	percent packet loss

# Select an appropriate version of ping and give any default options

# Example for Eric Wassenaar's ping
# (available from ftp://ftp.nikhef.nl/pub/network/ping.tar.Z)
# 
# This is the preferred version of ping because of the '-F' option.
# 
#	10 packets transmitted, 10 packets received, 0% packet loss
#	round-trip (ms) min/avg/max = 226/257/329 (std = 27.1)
# 
PING="/usr/sbin/eping -n -F -t 1"
PINGRTTMINFIELD=3
PINGRTTAVGFIELD=4
PINGRTTMAXFIELD=5
PACKETOPTS="END"

# Example for BSD
# 
# Most versions of BSD ping don't have a "fast" option so be careful
# how many hosts you graph with this (or reduce the PKCOUNT below and
# suffer from lower-resolution loss values)!  If you are in a highly
# trusted environment and you can run cricket as root then you could
# add the '-i 0.01' option to simulate flood ping speeds....
# 
# new:
#	5 packets transmitted, 5 packets received, 0.0% packet loss
#	round-trip min/avg/max/stddev = 14.757/45.524/795.846/29.792 ms
#
#PING="/sbin/ping -n -w 1"
#PINGRTTMINFIELD=4
#PINGRTTAVGFIELD=5
#PINGRTTMAXFIELD=6
#PACKETOPTS="FLAGS"
#
# old:
#	11 packets transmitted, 10 packets received, 9% packet loss
#	round-trip min/avg/max = 80.919/101.680/142.769 ms
# 
#PING="/sbin/ping -n -w 1"
#PINGRTTMINFIELD=3
#PINGRTTAVGFIELD=4
#PINGRTTMAXFIELD=5
#PACKETOPTS="FLAGS"

# Example for Solaris
# 
# Solaris ping doesn't have a "fast" option so be careful how many
# hosts you graph with this!  Get Eric Wassenaar's ping and use it!
# 
#	5 packets transmitted, 5 packets received, 0% packet loss
#	round-trip (ms)  min/avg/max = 68/72/80
# 
#PING="/usr/sbin/ping -n"
#PINGRTTMINFIELD=3
#PINGRTTAVGFIELD=4
#PINGRTTMAXFIELD=5
#PACKETOPTS="END"

# pick your fastest, smallest, awk:
#
#AWK=awk
#AWK=nawk
#AWK=gawk
AWK=mawk

PKCOUNT=10
PKSIZE=56

argv0=`basename $0`

if [ $# -ne 1 ] ; then
	echo "Usage: $argv0 host|IP"
	exit 2
fi

HOST=$1

# 
case $PACKETOPTS in
FLAGS)
	PING_CMD="$PING -c $PKCOUNT -s $PKSIZE $HOST"
	;;
END)
	PING_CMD="$PING $HOST $PKSIZE $PKCOUNT"
	;;
*)
	echo "$argv0: internal configuration error!" 1>&2
	exit 1
	;;
esac

$PING_CMD 2>/dev/null |
	$AWK '
		BEGIN {
			FS="/";
			minrtt = "";
			avgrtt = "";
			maxrtt = "";
			prcntloss = "";
		}
		/.* [0-9.]*% packet loss/ {
			narr = split($0, lossarr, ",");
			sub(/% packet loss/, "", lossarr[narr]);
			prcntloss = lossarr[narr] + 0;
		}
		NF > 1 && $1 ~ /^round-trip/ {
			minrtt = $'${PINGRTTMINFIELD}';
			sub(/[^0-9.]* = /, "", minrtt);
			avgrtt = $'${PINGRTTAVGFIELD}';
			maxrtt = $'${PINGRTTMAXFIELD}';
			sub(/ .*$/, "", maxrtt);
		}
		END {
			if (minrtt == "") {
				minrtt = "U";
			}
			if (avgrtt == "") {
				avgrtt = "U";
			}
			if (maxrtt == "") {
				maxrtt = "U";
			}
			printf("%s\t%s\n", minrtt, "minimum rtt in ms");
			printf("%s\t%s\n", avgrtt, "average rtt in ms");
			printf("%s\t%s\n", maxrtt, "maximum rtt in ms");
			printf("%s\t%s\n", prcntloss, "percent packet loss");
		}
	'
exit 0
